package nu.zoom.dsl.ast;

import nu.zoom.dsl.parser.EndpointsBaseVisitor;
import nu.zoom.dsl.parser.EndpointsParser;
import org.antlr.v4.runtime.tree.TerminalNode;

import java.util.ArrayList;
import java.util.List;

public class EndpointsVisitorTransformer extends EndpointsBaseVisitor<EndpointsParser.DocumentContext> {
    private final ArrayList<EndpointNode> endpoints = new ArrayList<>();
    private final ArrayList<ConfigItemNode> config = new ArrayList<>();
    private final ArrayList<CompoundTypeNode> dataTypes = new ArrayList<>();

    public EndpointsVisitorTransformer() {
    }

    public List<EndpointNode> getEndpoints() {
        return List.copyOf(endpoints);
    }

    public List<ConfigItemNode> getConfig() {
        return List.copyOf(config);
    }

    public List<CompoundTypeNode> getDataTypes() {
        return List.copyOf(dataTypes);
    }

    @Override
    public EndpointsParser.DocumentContext visitConfigitem(EndpointsParser.ConfigitemContext ctx) {
        String configKey = ctx.configkey().IDENTIFIER().getText();
        String configValue = getText(ctx.configvalue().IDENTIFIER(), ctx.configvalue().VALUE());
        this.config.add(new ConfigItemNode(configKey, configValue));
        return super.visitConfigitem(ctx);
    }

    @Override
    public EndpointsParser.DocumentContext visitCompoundType(EndpointsParser.CompoundTypeContext ctx) {
        this.dataTypes.add(extractCompoundTypeNode(ctx));
        return super.visitCompoundType(ctx);
    }

    @Override
    public EndpointsParser.DocumentContext visitEndpoint(EndpointsParser.EndpointContext ctx) {
        List<String> segments = ctx
                .path()
                .pathSegment()
                .stream()
                .map(
                        segment -> getText(segment.IDENTIFIER(), segment.VALUE())
                ).toList();
        TerminalNode typeReference = ctx.IDENTIFIER();
        if (typeReference != null) {
            var endpoint = new EndpointNode(new PathsNode(segments), typeReference.getText());
            this.endpoints.add(endpoint);
        } else {
            var compoundTypeNode = extractCompoundTypeNode(ctx.compoundType());
            var endpoint = new EndpointNode(new PathsNode(segments), compoundTypeNode.name());
            //this.dataTypes.add(compoundTypeNode);
            this.endpoints.add(endpoint);
        }
        return super.visitEndpoint(ctx);
    }

    private CompoundTypeNode extractCompoundTypeNode(EndpointsParser.CompoundTypeContext ctx) {
        String typeName = ctx.compoundTypeName().getText();
        List<FieldNode> fields = extractTypeFields(ctx.compoundFields().compoundField());
        return new CompoundTypeNode(typeName, fields);
    }

    private List<FieldNode> extractTypeFields(List<EndpointsParser.CompoundFieldContext> compoundFieldContexts) {
        return compoundFieldContexts.stream().map(
                ctx -> new FieldNode(
                        ctx.fieldName().getText(),
                        ctx.fieldType().getText()
                )
        ).toList();
    }

    // Concatenate the text from to terminal nodes. Useful for contexts that are either an identifier or a value,
    // and you just want the text from whichever is not null.
    private String getText(TerminalNode identifier, TerminalNode value) {
        return
                ((identifier != null) ? identifier.getText() : "") +
                        ((value != null) ? value.getText() : "");
    }
}
